{% extends BASE_TEMPLATE %}

{% load staticfiles %}

{% block head-title %}{{ title }}{% endblock %}

{% block main %}
<link rel="stylesheet" type="text/css" href="{% static 'bom/style.css' %}" />
<div class="container">
    <h1 class="col s12 light header center">Error</h1>
</div>
{% endblock %}

{% block menu %}
{% endblock%}

{% block content %}
<!-- 
<ul class="messages text-center">
    {% if messages %}
    {% for message in messages %}
    <li{% if message.tags %} class="{{ message.tags }}"{% endif %}>
        {{ message.message }}
    </li>
    {% endfor %}
    {% else %}
    <li class="ERROR">
        An unknown error has occured.
    </li>
    {% endif %}
</ul>
 -->

{% endblock %}