{% extends BASE_TEMPLATE %}

{% load materializecss %}
{% load staticfiles %}

{% block head-title %}{{ title }}{% endblock %}

{% block main %}
    <link rel="stylesheet" type="text/css" href="{% static 'bom/style.css' %}"/>
{% endblock %}

{% block menu %}
    <li><a href="javascript:history.back()">Cancel</a></li>
{% endblock %}

{% block content %}
    <div class="container-app">
    {% if h1 %}<h2 class="light" style="margin-top: 0px;">{{ h1 }}</h2>{% endif %}
    <form name="seller" action="{{ action }}" method="post" class="col s12">
        <div class="row">
            {% csrf_token %}
            {{ form|materializecss:'m6 s12' }}
        </div>
        <div class="row">
            <div class="col s6">
                <a href="javascript:history.back()" class="waves-effect waves-light btn-flat grey-text lighten-1" style="margin-left: -16px;">Cancel</a>
            </div>
            <div class="col s6 right-align">
                <button class="waves-effect waves-light btn green lighten-1" type="submit" name="action">Save</button>
            </div>
        </div>
    </form>
    </div>
{% endblock %}